
/*
 *  render.cpp
 *  Eleonora Maria Irene Oreggia ((xname))
 *  This is REBUS, good luck with it!!  
 *  Thanks to Andrew McPherson, Giulio Moro and the Bela team (bela.io).
 * 	Developed at Queen Mary University of London, at Media & Arts Technology DTC
 *	A collaboration between the Antennas & Electromagnetics Group and C4DM (Centre for Digital Music)
 */



#include <Bela.h>
#include <cmath>
#include <unistd.h>


float gPhase = 0;
float gInverseSampleRate;
float gFrequency = 440;

float gEmGain = 1;
float gEmPhase= 0;

float gSoundPressure = 0.00; 

float gMinPhase = 0.2;
float gMaxPhase = 1.8;
float gMinGain = 0.3;
float gMaxGain = 1.4;

float gMinFrequency = 20.0;
float gMaxFrequency = 2000.0;

float gADCFullScale = 4.096;

float gAmplitude;

// High Pass Coefficients
float gHP_a1 = 0.0;
float gHP_a2 = 0.0;
float gHP_b0 = 0.0;
float gHP_b1 = 0.0;
float gHP_b2 = 0.0;

// store states High Pass
float prevStateInHP = 0.0;  //gHP_b1
float pastStateInHP = 0.0;  //gHP_b2
float prevStateOutHP = 0.0;  //gHP_a1
float pastStateOutHP = 0.0;  //gHP_a2


bool setup(BelaContext *context, void *userData)
{
	/*
	 * Calculating filter coefficients
	 * default is 20Hz
	 */

	float filterFrequency = 300;
	float Q = sqrt(2.0)*0.5; // quality factor for Butterworth filter
	float T = 1/context->audioSampleRate; //period T, inverse of sampling frequency
	float w = 2.0*M_PI*filterFrequency; //angular freq in radians per sec

	float delta = (4.0+((2*T)*(w/Q))+(w*w*T*T)); //factor to simplify the equation

	gInverseSampleRate = 1.0 / context->audioSampleRate;
	
	return true;

}


void render(BelaContext* context, void* arg)
{

	for(unsigned int n = 0; n < context->audioFrames; n++) {

		// read the  PHASE 
		
		float phaseReading = context->analogIn[n/2*8]; 
		float phaseVoltage = phaseReading * gADCFullScale;  //4.096 full scale value of the converter of matrixIn
		gEmPhase = (phaseVoltage - gMinPhase) / (gMaxPhase - gMinPhase);
		gPhase = (- gEmPhase * 100 +180)/180 * M_PI; // negative slope therefore minus, approx 0.03 to 0

		// read the  GAIN
		float gainReading = context->analogIn[n/2*8 + 4]; // interleaved, half audio rate
		float gainVoltage = gainReading *  gADCFullScale;
		gEmGain =  gainVoltage * (gMaxGain - gMinGain) + gMinGain;
		gAmplitude =  20*log10(gEmGain) * (gMaxGain - gMinGain) + gMinGain;
		//gAmplitude =  (20*log10(gEmGain) - gMinGain) / (gMaxGain - gMinGain);

//		gFrequency = gPhase*(gMaxFrequency - gMinFrequency) + gMinFrequency;	
		gFrequency = map(gainReading, gMinGain, gMaxGain, gMinFrequency, gMaxFrequency);

		gPhase += 2.0 * M_PI * gFrequency / context->audioSampleRate;
		
		
			// stay in range
		if(gPhase >= 2.0 * M_PI)
	            gPhase -= 2.0 * M_PI;
//		while(gPhase < 0)
//	            gPhase +=2.0 * M_PI;
	            

//		gPhase = map(gEmPhase, gMinPhase, gMaxPhase, );

		float out = gAmplitude * sin(gPhase);








//		float out=(gHP_b0*filterIn) + (gHP_b1*prevStateInHP) + (gHP_b2*pastStateInHP) - (gHP_a1*prevStateOutHP) - (gHP_a2*pastStateOutHP);

		// rt_printf("filterIn is  %f\n", filterIn);

//		pastStateInHP=prevStateInHP;    //store current x[n-1] that will become x[n-2]
//		prevStateInHP=filterIn;			//store current x[n] that will become x[n-1]

//		pastStateOutHP=prevStateOutHP;  //store current y[n-1] that will become y[n-1]
//		prevStateOutHP=out; //store current y[n] that will become y[n-1]

		out/=2;

		// gPhase += 2.0 * M_PI * gFrequency * gInverseSampleRate;

		if(gEmPhase > 2.0 * M_PI)
			gEmPhase -= 2.0 * M_PI;

		for(unsigned int channel = 0; channel < context->audioInChannels; channel++)
//			context->audioOut[n * context->audioOutChannels + channel] = out;
			audioWrite(context, n, channel, out);

	}
}


void cleanup(BelaContext *context, void *userData)
{

}






